<style>
	input:focus { 
		    background-color: green;
		}
</style>
<form class="form-horizontal form-complex">
{% for param in mod_alert.parameters %}
    {% with mod_alert=mod_alert, param=param %}
        {% include param.format_type + ".html" %}
    {% endwith %}
{% endfor %}
</form>
