<form class="form-horizontal form-complex">
<table>
    <tr>
        {% for param in mod_alert.parameters %}
            <td>
            {% with mod_alert=mod_alert, param=param %}
                {% include param.format_type + ".html" %}
            {% endwith %}
            </td>
        {% endfor %}
    </tr>
</table>
</form>
